import os
import glob
import pandas as pd
import numpy as np
import argparse

############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-i",
                    help="result folder of buty_phyl prediction", type=str,
                    default='ButyPhyl',metavar='ButyPhyl')
parser.add_argument("-r",
                    help="result folder of the summary", type=str,
                    default='ButyPhyl_summary',metavar='ButyPhyl_summary')

################################################## Definition ########################################################
args = parser.parse_args()
try:
    os.mkdir(args.r)
except OSError:
    pass

################################################### Function ########################################################
def read_species(filename):
    return pd.read_csv(filename, sep='\t',index_col=0,header=0)

def merge_pathogen_commensal(df1, df2):
    filename = df1.columns[0] + '.sum'
    temp_species_df = pd.merge(df1, df2,
    how='outer', left_index = True, right_index = True)
    return pd.DataFrame(temp_species_df.sum(axis = 1), columns = [filename])

def add_pathogen_commensal(df1,df2):
    temp_abu_df = df1
    temp_abu_df['Abu_with_Traits'][0] = df1['Abu_with_Traits'][0] + df2['Abu_with_Traits'][0]
    return temp_abu_df

################################################### Programme #######################################################
result_species = glob.glob(os.path.join(args.i,'*.bpbspecies.commensal.abu'))
result_abu = glob.glob(os.path.join(args.i,'*.infertraits.commensal.abu'))

if result_species != []:
    i = 0
    for commensal_species in result_species:
        pathogen_species = commensal_species.replace('commensal','pathogen')
        pathogen_species_df = read_species(pathogen_species)
        commensal_species_df = read_species(commensal_species)
        if i == 0:
            all_species_df = merge_pathogen_commensal(pathogen_species_df, commensal_species_df)
        else:
            all_species_df = pd.merge(all_species_df,
            merge_pathogen_commensal(pathogen_species_df, commensal_species_df),
            how='outer', left_index = True, right_index = True)
        i += 1
    all_species_df.to_csv(os.path.join(args.r, 'all.bpbspecies.abu'), sep='\t',
                  header=True)

if result_abu != []:
    i = 0
    for commensal_abu in result_abu:
        pathogen_abu = commensal_abu.replace('commensal','pathogen')
        pathogen_abu_df = read_species(pathogen_abu)
        commensal_abu_df = read_species(commensal_abu)
        if i == 0:
            all_abu_df = add_pathogen_commensal(pathogen_abu_df, commensal_abu_df)
        else:
            all_abu_df = all_abu_df.append(add_pathogen_commensal(pathogen_abu_df, commensal_abu_df))
        i += 1
    all_abu_df.to_csv(os.path.join(args.r, 'all.bpb.abu'), sep='\t',
                  header=True)



import os
from Bio import SeqIO
import argparse
import glob
import pandas as pd
import numpy as np

############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-t",
                    help="file name of your otu_table", type=str, default='a.otu.table',metavar='a.otu.table')
parser.add_argument("-s",
                    help="file name of your otu_seq", type=str, default='a.otu.fasta',metavar='a.otu.fasta')
parser.add_argument("-r",
                    help="results_dir", type=str, default='Filtered_OTU',metavar='Filtered_OTU')
parser.add_argument("-top",
                    help="number of top OTUs", type=int, default=5000,metavar=5000)


################################################## Definition ########################################################
args = parser.parse_args()
try:
    os.mkdir(args.r)
except OSError:
    pass


################################################### Function ########################################################
def Maxabu(Abus):
    Abumax=0
    for number in Abus:
        Abumax=max(Abumax, int(number.replace('\r','').replace('\n','')))
    return Abumax


def biomTOcsv(otutablefile):
    os.system('biom convert -i ' + otutablefile +
    ' -o ' + otutablefile + '.otu_table.temp.txt --to-tsv')
    os.system('sed 1d ' + otutablefile + '.otu_table.temp.txt > '
    + otutablefile + '.otu_table.txt')
    os.system('rm -rf ' + otutablefile + '.otu_table.temp.txt')
    return otutablefile + '.otu_table.txt'


def Tableinput(otutablefile,topnumber):
    filenames, fileextensions = os.path.splitext(otutablefile)
    if fileextensions == '.biom':
        otutablefile = biomTOcsv(otutablefile)
    OTUtable=dict()
    OTUtop=[]
    rootofutb, otutable = os.path.split(otutablefile)
    OTU_table = pd.read_csv(otutablefile, sep='\t', index_col=0)
    #OTU_table['sum']=OTU_table.sum(axis=0,numeric_only=True)
    OTU_table_new=OTU_table.div(OTU_table.sum(axis=0,numeric_only=True), axis=1)
    OTU_table_new.to_csv(os.path.join(args.r,otutable+'.abu.table'), sep='\t', header=True)
    Abus=OTU_table_new.max(axis=1,numeric_only=True).tolist()
    i=0
    for OTUs in OTU_table_new.index:
        OTUtable.setdefault(OTUs,Abus[i])
        i+=1
    Abus.sort(reverse=True)
    f1 = open(os.path.join(args.r,otutable+'.max.abu'),'w')
    for OTUs in OTUtable:
        try:
            if OTUtable[OTUs] >= Abus[topnumber-1]:
                f1.write(str(OTUs)+'\t'+str(OTUtable[OTUs])+'\n')
                OTUtop.append(str(OTUs))
        except IndexError:
            f1.write(str(OTUs) + '\t' + str(OTUtable[OTUs]) + '\n')
            OTUtop.append(str(OTUs))
    f1.close()
    return OTUtop


def Tableoutput(OTUtop,otuseqfile):
    rootofus, otuseq = os.path.split(otuseqfile)
    f1 = open(os.path.join(args.r,otuseq+'.filter'),'w')
    Write = 0
    for record in SeqIO.parse(open(otuseqfile, 'r'), 'fasta'):
        if str(record.id) in OTUtop:
            Write = 1
            f1.write('>' + str(record.id) + '\n' + str(str(record.seq)) + '\n')
    f1.close()
    if Write == 0:
        print('WRANING: please make sure the OTU IDs/names in OTU table and OTU seqs are the same!!!\n')
        print('If you see this warning, please check your input files, delete the result folder and re-run the scripts.')
################################################### Programme #######################################################
Tableoutput(Tableinput(args.t,args.top),args.s)



import argparse
from Bio import AlignIO
import os
from Bio import SeqIO


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-s",
                    help="file name of your otu_seq", type=str, default='a.otu.fasta',metavar='a.otu.fasta')
parser.add_argument("-r",
                    help="results_dir", type=str, default='ButyPhyl',metavar='ButyPhyl')
parser.add_argument("-rd",
                    help="the reference data of gene traits", type=str, default='Data.txt',metavar='Data.txt')
parser.add_argument("-a",
                    help="file name of alignment", type=str, default='None',metavar='a.align.16S')


################################################## Definition ########################################################
args = parser.parse_args()


################################################### Function ########################################################
def Alignname_format(filename):
    Alignroot, Alignfile = os.path.split(filename)
    f1 = open(os.path.join(Alignroot, Alignfile + '.format'), 'w')
    f2 = open(os.path.join(Alignroot, Alignfile + '.format.name'), 'w')
    for record in SeqIO.parse(open(filename, 'r'), 'fasta'):
        f1.write('>' + str(record.id).replace('_R_','') + '\n' + str(record.seq) + '\n')
        f2.write(str(record.id).replace('_R_','')+'\t'+str(record.id).replace('_R_','')+'\n')
    f1.close()
    f2.close()


################################################### Programme #######################################################
if args.a == 'None':
    rootofus, otuseq = os.path.split(args.s)
    f1=open(os.path.join(args.r, otuseq+'.format'),'w')
    f2=open(os.path.join(args.r, otuseq+'.taxa.translate'),'w')
    os.system('cp '+args.rd+' '+os.path.join(args.r, otuseq+'.data'))
    Refdata=[]
    for lines in open(os.path.join(args.r, otuseq+'.data'),'r'):
        Refdata.append(lines.split('\t')[0])
    f3=open(os.path.join(args.r, otuseq+'.data'),'ab')
    f2.write('#NEXUS\nBegin trees;\n\ttranslate')
    i=1
    for record in SeqIO.parse(open(os.path.join(args.r,args.s), 'r'), 'fasta'):
        f1.write('>' + str(i) + '\n' + str(record.seq) + '\n')
        if i==1:
            f2.write('\n')
        else:
            f2.write(',\n')
        f2.write('\t\t'+str(i)+'\t'+str(record.id))
        if str(record.id) not in Refdata:
            f3.write(str(record.id)+'\t?\n')
        i+=1
    f2.write(';\n\t\ttree tree.taxa = ')
    f1.close()
    f2.close()
    f3.close()
else:
    Alignname_format(args.a)